@@ -1,6 +1,9 @@
 import logging
+import random
+
 from pyage.core.address import Addressable
-from pyage.core.inject import Inject
+from pyage.core.inject import Inject, InjectWithDefault
+
 
 logger = logging.getLogger(__name__)
 
@@ -13,6 +16,7 @@ class EmasAgent(Addressable):
         self.genotype = genotype
         self.energy = energy
         self.steps = 0
+        self.dead = False
         self.evaluation.process([genotype])
 
     def step(self):
@@ -26,8 +30,10 @@ class EmasAgent(Addressable):
                     self.emas.reproduce(self, neighbour)
                 else:
                     self.meet(neighbour)
-                if self.emas.can_migrate(self):
-                    self.migration.migrate(self)
+                if self.emas.can_migrate(self) and self.migration.migrate(self):
+                    self.locator.remove_agent(self)
+                elif self.emas.should_move(self):
+                    self.move()
         except:
             logging.exception('')
 
@@ -64,11 +70,12 @@ class EmasAgent(Addressable):
     def death(self):
         self.distribute_energy()
         self.energy = 0
+        self.dead = True
         self.parent.remove_agent(self)
         logger.debug(str(self) + "died!")
 
     def distribute_energy(self):
-        logger.debug("death, energy level: %d" % self.energy)
+        logger.debug("%s is dying, energy level: %d" % (self, self.energy))
         if self.energy > 0:
             siblings = set(self.parent.get_agents())
             siblings.remove(self)
@@ -84,27 +91,49 @@ class EmasAgent(Addressable):
                 siblings.pop().add_energy(e)
                 left -= e
 
+    def move(self):
+        allowed_moves = self.locator.get_allowed_moves(self)
+        if allowed_moves:
+            self.locator.remove_agent(self)
+            destination = get_random_move(allowed_moves)
+            self.locator.add_agent(self, destination)
+            logger.debug("%s moved to %s" % (self, destination))
+
+    def __repr__(self):
+        return "<EmasAgent@%s>" % self.get_address()
+
 
 class EmasService(object):
-    @Inject("minimal_energy", "reproduction_minimum", "migration_minimum", "newborn_energy")
+    @Inject("minimal_energy", "reproduction_minimum", "migration_minimum", "newborn_energy", "locator")
+    @InjectWithDefault(("move_probability", 0.1))
     def __init__(self):
         super(EmasService, self).__init__()
 
     def should_die(self, agent):
-        return agent.get_energy() <= self.minimal_energy
+        return agent.get_energy() <= self.minimal_energy and not agent.dead
 
     def should_reproduce(self, a1, a2):
-        return a1.get_energy() > self.reproduction_minimum and a2.get_energy() > self.reproduction_minimum
+        return a1.get_energy() > self.reproduction_minimum and a2.get_energy() > self.reproduction_minimum \
+               and self.locator.get_allowed_moves(a1)
 
     def can_migrate(self, agent):
         return agent.get_energy() > self.migration_minimum and len(agent.parent.get_agents()) > 10
 
+    def should_move(self, agent):
+        return random.random() < self.move_probability
+
     def reproduce(self, a1, a2):
-        logger.debug(str(a1) + " " + str(a2) + "reproducing!")
+        logger.debug(str(a1) + " " + str(a2) + " reproducing!")
         energy = self.newborn_energy / 2 * 2
         a1.energy -= self.newborn_energy / 2
         a2.add_energy(-self.newborn_energy / 2)
         genotype = a1.crossover.cross(a1.genotype, a2.get_genotype())
         a1.mutation.mutate(genotype)
-        a1.parent.add_agent(EmasAgent(genotype, energy))
+        newborn = EmasAgent(genotype, energy)
+        a1.parent.add_agent(newborn)
+        self.locator.add_agent(newborn, get_random_move(self.locator.get_allowed_moves(a1)))
 
+
+def get_random_move(allowed_moves):
+    destination = random.sample(allowed_moves, 1)[0]
+    return destination
\ No newline at end of file
